      $set comp noseg noalter noqual
      *
      *****************************************************************
      * Program: CALLFDEM.CBL
      *
      * Function: Demonstrate how to use the callable external file
      * 	   handler from COBOL/2.
      *
      * Written by: Micro Focus, Inc.
      *
      * Date written: April 1989.
      *
      * Tested with COBOL/2 V1.1.18.
      *
      * Description:
      * 	     Program first will delete an ISAM file, if it
      * 	      exists.
      * 	     Program will create an ISAM file. This ISAM file
      * 	      will have a numeric prime key (10 characters
      * 	      long) and an alphanumeric alternate key that
      * 	      allows duplicates. After file is created, program
      * 	      writes 10 records. Then, program will do a direct
      * 	      read on the primary key, then do a direct read of
      * 	      an alternate key.
      *
      *****************************************************************
      *
       working-storage section.
      *
      * Output file name and length
      *
       01  file-name	      pic x(12) value "TSTEXTFH.DAT".
       78  file-name-len      value 12.

       01  FILE-HANDLER-FUNCTION.
	   03  FILLER	      pic x value x"fa".
	   03  EXTFH-OP-CODE  pic x.
      *
      * OP CODES for the file handler
      *
       78 fhop-delete-file    value x"F8".
       78 FHOP-OPEN-I-O       VALUE X"02".
       78 FHOP-CLOSE	      VALUE X"80".
       78 FHOP-WRITE	      VALUE X"F3".
       78 fhop-read-random    VALUE X"F6".
      *
      * Data record. Note the 4 character filler at the end of the
      *  record. Every file used with the external file handler must
      *  have a 4 character filler at the end of the record. This
      *  filler item is used the the external file handler for internal
      *  processing. When we WRITE the record, this 4 character filler
      *  will not be written to the file.
      *
       01  record-area.
	   05  prime-key      pic 9(10).
	   05  alt-key	      pic x(10).
	   05  filler	      pic x(4).
      *
      * Length of the data record (excluding the 4 character filler).
      *
       78  file-1-rec-len     value 20.
      *
      * Temporary holding area for the file pointer returned by
      *  the external file handler.
      *
       01  ws-hold-file-handle usage pointer.
      *
      /
      *****************************************************************
      *
      * Parameter block
      *
       01  parameter-block.
	   05  FCD-FILE-STATUS.
	       10  fcd-status-key-1 pic x.
	       10  fcd-status-key-2 pic x.
	       10  fcd-binary redefines fcd-status-key-2
				    pic 99 comp-x.
	   05  FILLER		    PIC X(3).
	   05  FCD-ORGANIZATION     PIC 9(2) COMP-X.
	   05  FCD-ACCESS-MODE	    PIC 9(2) COMP-X.
	   05  FCD-OPEN-MODE	    PIC 9(2) COMP-X.
	   05  FILLER		    PIC X(3).
	   05  FCD-NAME-LENGTH	    PIC 9(4) COMP-X.
	   05  FILLER		    PIC X(11).
	   05  FCD-LOCK-MODE	    PIC 9(2) COMP-X.
	   05  FCD-OTHER-FLAGS	    PIC 9(2) COMP-X.
	   05  FILLER		    PIC X(2).
	   05  FCD-HANDLE	    USAGE POINTER.
	   05  FILLER		    PIC X.
	   05  FCD-STATUS-TYPE	    PIC 9(2) COMP-X.
	   05  FCD-FILE-FORMAT	    PIC 9(2) COMP-X.
	   05  FILLER		    PIC X(3).
	   05  FCD-MAX-REC-LENGTH   PIC 9(4) COMP-X.
	   05  FILLER		    PIC X(3).
	   05  FCD-RELATIVE-KEY     PIC 9(9) COMP-X.
	   05  FCD-RECORDING-MODE   PIC 9(2) COMP-X.
	   05  FCD-CURRENT-REC-LEN  PIC 9(4) COMP-X.
	   05  FCD-MIN-REC-LENGTH   PIC 9(4) COMP-X.
	   05  FCD-KEY-ID	    PIC 9(4) COMP-X.
	   05  FCD-LINE-COUNT	    REDEFINES FCD-KEY-ID
				    PIC 9(4) COMP-X.
	   05  FCD-KEY-LENGTH	    PIC 9(4) COMP-X.
	   05  FCD-RECORD-ADDRESS   USAGE POINTER.
	   05  FCD-FILENAME-ADDRESS USAGE POINTER.
	   05  FCD-KEY-DEF-ADDRESS  USAGE POINTER.
	   05  FILLER		    PIC X(14).
      /
      *****************************************************************
      *
      * Key Defination Area
      *
       01 key-definition-area.
      *
      * Global Information Area. 14 bytes in length.
      *
	   05 GLOBAL-INFORMATION-AREA.
	      10 LENGTH-OF-KEY-DEF-AREA   PIC 9(4)  COMP-X.
	      10 FILLER 		  PIC X(4).
	      10 NUMBER-OF-KEYS 	  PIC 9(4)  COMP-X.
	      10 FILLER 		  PIC X(6).
      *
      * Key Definition Parameter Block Area.
      *
      * Since this ISAM file has 2 keys, we must define 2 Key
      *  Definition Parameter Block Areas. Each block is 16 bytes
      *  long.
      *
	   05  key-definition-parameter-block  occurs 2 times.
	       10 COMPONENT-COUNT     PIC 9(4) COMP-X.
	       10 OFFSET-TO-COMPONENT PIC 9(4) COMP-X.
	       10 KEY-FLAGS	      PIC 9(2) COMP-X.
	       10 KEY-COMPRESSION     PIC 9(2) COMP-X.
	       10 SPARSE-CHARACTER    PIC X.
	       10 FILLER	      PIC X(9).
      *
      * Component Definition Area. Each area is 10 bytes in length.
      *  We need to define 2 areas because we have 2 keys (1 prime and
      *  1 alternate).
      *
	   05  component-definition-area  occurs 2.
	       10 FILLER		  PIC X(2).
	       10 OFFSET-TO-COMPONENT-DEF PIC 9(9) COMP-X.
	       10 COMPONENT-LENGTH	  PIC 9(9) COMP-X.
      *
      *****************************************************************
      *
       procedure division.
      *
      *****************************************************************
      *
       start-program section.
	   perform delete-isam-file.
	   perform open-isam-file.
	   perform write-records
	   perform read-prime-key.
	   perform read-alt-key.
	   perform close-files
	   stop run.
      *
      *****************************************************************
      *
       delete-isam-file section.
      *
      *****************************************************************
      *
      * Function: This section will delete the ISAM file "TSTEXTFH",
      * 	   if it exists. Because we will be creating this file,
      * 	   we want to be sure it doesn't exist. (Note - the
      * 	   DELETE function will delete both the DATA and IDX
      * 	   portions of the ISAM file.)
      *
      *****************************************************************
      *
      * Set all fields to binary zeros.
      *
	   move low-values to parameter-block
      *
      * Setup length of the filename to delete
      *
	   move file-name-len to fcd-name-length
      *
      * Setup the address to the file name
      *
	   set fcd-filename-address to address of file-name
      *
      * Tell the file handler that the file is an ISAM file.
      *
	   move 2 to fcd-organization
      *
      * Tell the file handler that the file is closed at the
      *  time of the call
      *
	   move 128 to fcd-open-mode
      *
      * Delete the ISAM file
      *
	   move fhop-delete-file to extfh-op-code
	   perform call-file-handler
	   if fcd-status-key-1 not = "0" and
	      fcd-binary       not = 013
		   display "External file handler error"
		   display "status = " fcd-file-status
		   display "Error during delete of the data file"
		   display "aborting"
		   stop run.
      *
      *****************************************************************
      *
       open-isam-file section.
      *
      *****************************************************************
      *
      * Function: Open indexed file with 1 prime key, 1 alternate key
      * 	   with duplicates allowed.
      *
      *****************************************************************
      *
      * Set all fields to binary zeros.
      *
	   move low-values to parameter-block
	   move low-values to key-definition-area
      *
      * Organization is INDEXED.
      *
	   move 2 to fcd-organization
      *
      * File is closed when we make our first call.
      *
	   move 128 to fcd-open-mode
      *
      * Setup length of the file name.
      *
	   move file-name-len to fcd-name-length
      *
      * Setup minimum and maximum record lengths.
      *
	   move file-1-rec-len to fcd-max-rec-length
	   move file-1-rec-len to fcd-min-rec-length
      *
      * Access mode is DYNAMIC
      *
	   move 8 to fcd-access-mode
      *
      * Setup addresses of the data record, file name and
      *  Key Definition Area.
      *
	   set fcd-record-address to address of record-area
	   set fcd-filename-address to address of file-name
	   set fcd-key-def-address to address of key-definition-area
      *
      *****************************************************************
      *
      * Setup the key information
      *
      * The Key Definition Area in this example will be 66 bytes
      *  long (Global Information Area is 14 bytes, we have 2 Key
      *  Definition Parameter Block Areas, each being 16 bytes
      *  long and 2 Component Definition Areas, each being 10
      *  bytes long).
      *
	   move 66 to length-of-key-def-area
      *
      * There are 2 keys to this file
      *
	   move 2 to number-of-keys
      *
      *****************************************************************
      *
      * Set prime key definition details
      *
      *****************************************************************
      *
      * Prime key is not a split key. It is made up of one elementary
      *  item.
      *
	   move 1 to component-count (1)
      *
      * Initialize offset to the first Component Definition Area
      *  (relative to the start of the Global Information Area),
      *  starting with 0.
      *
      * The Global Information Area is 14 bytes long. The Key
      *  Definition Parameter Block area is 16 bytes long. We have 2
      *  2 of these. This is a total offset of 46 bytes.
      *
	   move 46 to OFFSET-TO-COMPONENT (1)
      *
      * This is the prime key that we are defining.
      *
	   move 16 to key-flags (1)
      *
      * Setup the offset to and length of the prime key. The prime key
      *  starts with the first character of the record (offset 0) and
      *  is 10 characters long.
      *
	   move 0 to offset-to-component-def (1)
	   move 10 to component-length (1)
      *
      *****************************************************************
      *
      * Setup the alternate key details.
      *
      *****************************************************************
      *
      * The alternate key is not a split key. It is made up of one
      *  elementary item.
      *
	   move 1 to component-count (2)
      *
      * Setup the offset, relative to the start of the Global
      *  Information Area, where the alternate key Component
      *  Definitions Area (for this key) is defined.
      *
      * The Global Information Area is 14 bytes long. Between the
      * Global Information Area and the Component Definitions Area for
      * this key are 2 Key Definition Parameter Block Areas (each is 16
      * bytes long) and there is a Component Definition Area for the
      * prime key (which is 10 bytes long). Therefore, the offset to
      * the alternate key's Component Definition Area is 56 bytes.
      *
	   move 56 to offset-to-component (2)
      *
      * This is an alternate key with duplicates that we are defining.
      *
	   move 64 to key-flags (2)
      *
      * Setup offset to and length of the alternate key. Our alternate
      *  key starts in the 11th character position of our record and is
      *  10 characters long. Remember that offset 0 means the first
      *  character position of the record.
      *
	   move 10 to offset-to-component-def (2)
	   move 10 to component-length (2)
      *
      * Open the file I-O.
      *
	   move fhop-open-i-o to extfh-op-code
      *
      * Call the external file handler.
      *
	   perform call-file-handler
	   if fcd-status-key-1 not = "0"
	       display "External file handler error"
	       display "status = " fcd-file-status
	       display "Error during open"
	       display "aborting"
	       stop run.
      *
      * Save the file handle returned.
      *
	   set ws-hold-file-handle to fcd-handle.
      *
      *****************************************************************
      *
	write-records section.
      *
      *****************************************************************
      *
      * Function: Writes 5 records to the ISAM file just defined.
      *
      *****************************************************************
      *
      * Write record function code
      *
	   move fhop-write to extfh-op-code
      *
	   move 1 to prime-key
	   move all "A" to alt-key
	   perform call-file-handler
	   if fcd-status-key-1 not = "0"
	       display "External file handler error"
	       display "status = " fcd-file-status
	       display "Error during first write"
	       display "aborting"
	       stop run.

	   move 2 to prime-key
	   move all "B" to alt-key
	   perform call-file-handler
	   if fcd-status-key-1 not = "0"
	       display "External file handler error"
	       display "status = " fcd-file-status
	       display "Error during second write"
	       display "aborting"
	       stop run.

	   move 3 to prime-key
	   move all "C" to alt-key
	   perform call-file-handler
	   if fcd-status-key-1 not = "0"
	       display "External file handler error"
	       display "status = " fcd-file-status
	       display "Error during third write"
	       display "aborting"
	       stop run.

	   move 4 to prime-key
	   move all "D" to alt-key
	   perform call-file-handler
	   if fcd-status-key-1 not = "0"
	       display "External file handler error"
	       display "status = " fcd-file-status
	       display "Error during fourth write"
	       display "aborting"
	       stop run.

	   move 5 to prime-key
	   move all "E" to alt-key
	   perform call-file-handler
	   if fcd-status-key-1 not = "0"
	       display "External file handler error"
	       display "status = " fcd-file-status
	       display "Error during fifth write"
	       display "aborting"
	       stop run.
      *
      *****************************************************************
      *
	read-prime-key section.
      *
      *****************************************************************
      *
      * Function: Reads the ISAM file along the prime key path.
      *
      *****************************************************************
      *
      * Clear the parameter block
      *
	   move low-values to parameter-block
      *
      * Setup the file handle for this file.
      *
	   set fcd-handle to ws-hold-file-handle
      *
      * Set the address to our record
      *
	   set fcd-record-address to address of record-area
      *
      * Organization is indexed
      *
	   move 2 to fcd-organization
      *
      * Read the ISAM record whose prime key = 3
      *
	   move 3 to prime-key
      *
      * Read from the prime key
      *
	   move zero to fcd-key-id
      *
      * Read random function code.
      *
	   move fhop-read-random to extfh-op-code
      *
      * Call the external file handler
      *
	   perform call-file-handler
	   if fcd-status-key-1 not = "0"
	       display "External file handler error"
	       display "status = " fcd-file-status
	       display "Error during prime key read"
	       display "aborting"
	       stop run.
      *
      * Display results
      *
	   display record-area.

      *****************************************************************
      *
	read-alt-key section.
      *
      *****************************************************************
      *
      * Function: Reads the ISAM file along the alternate key path.
      *
      *****************************************************************
      *
      * Clear the parameter block
      *
	   move low-values to parameter-block
      *
      * Setup the file handle for this file.
      *
	   set fcd-handle to ws-hold-file-handle
      *
      * Set the address to our record
      *
	   set fcd-record-address to address of record-area
      *
      * Organization is indexed
      *
	   move 2 to fcd-organization
      *
      * Read the ISAM record whose alternate key = all "E"
      *
	   move all "E" to alt-key
      *
      * Read from the alternate key
      *
	   move 1 to fcd-key-id
      *
      * Read random function code.
      *
	   move fhop-read-random to extfh-op-code
      *
      * Call the external file handler
      *
	   perform call-file-handler
	   if fcd-status-key-1 not = "0"
	       display "External file handler error"
	       display "status = " fcd-file-status
	       display "Error during alternate key read"
	       display "aborting"
	       stop run.
      *
      * Display results
      *
	   display record-area.
      *
      *****************************************************************
      *
       call-file-handler section.
      *
      *****************************************************************
      *
      * Function: Call the external file handler
      *
      *****************************************************************
      *
	   call "EXTFH" using file-handler-function parameter-block.
      *
      *****************************************************************
      *
       close-files section.
      *
      *****************************************************************
      *
      * Function: Close the file.
      *
      *****************************************************************
      *
	   move fhop-close to extfh-op-code
	   perform call-file-handler.
	   if fcd-status-key-1 not = "0"
	       display "External file handler error"
	       display "status = " fcd-file-status
	       display "Error during close"
	       display "aborting"
	       stop run.

