      $SET COMP NOQUAL NOALTER NOOSVS VSC2 ANS85
      *************************************************************************
      * COBOL Concurrent Thread Demonstration Program
      *
      * This software is the Copyright of Micro Focus Ltd, 1989
      *
      *    NOTE: the term "thread" is used throughout the demo programs,
      *    this term is used to indicate a seperate thread of execution
      *    within the Run Time Sytem (RTS) which controls the execution
      *    of the COBOL programs. A main COBOL program and its called
      *    subprograms constitute a "run unit" and this term is used in
      *    existing Workbench documentation for concurrent COBOL
      *    program operations. A run unit exists in each thread of
      *    execution.
      *
      * This program is the controlling thread of a very simple
      * demonstration of the use of the multi-threading support
      * routines provided by COBENV.
      *
      * COBENV is a file that when called installs the routines
      * contained within the file. These routines are then available
      * for use by this and any other created thread.
      *
      * The demonstration programs try to establish three threads
      * including the thread containing this program.
      *
      *                       Ŀ
      *                       Thread 0
      *                       CONDEMO 
      *                       
      *              Ŀ
      *          Ŀ               Ŀ
      *          Thread 1               Thread 2
      *          CONDEMOA               CONDEMOB
      *            CALL                 (Clock) 
      *                         
      *              
      *          Ŀ
      *          CONDEMOC
      *          (TICTAC)
      *          
      *
      * The demonstration programs themselves do not indicate how a
      * multi-threading system should be set up but do show how the
      * functions can be used and what sort of features they provide.
      *
      * The main thing to be aware of is that you should try and ensure
      * that each thread is performing a particular function to avoid
      * contention over machine resources eg. access to keyboard and
      * screen. In these demonstration programs each thread is unaware
      * of the other threads writing to the screen so screen clear
      * operations in one thread destroy information displayed by
      * another thread. This provides unpleasant flashes on the
      * display.
      *
      * Another important factor is closing down the system. When the
      * user interface thread decides it is time to call it a day then
      * a termination request must be made on the other threads to
      * allow return to whatever invoked you (eg Workbench or DOS). If
      * the user interface thread terminates (STOP RUN or RTS error)
      * without closing down the other threads then you have no control
      * over the system and a reboot is required to recover from this
      * deadlock.
      *
      * The two areas on which to concentrate initially are the
      * establishment of a message segment and "put"ing and "get"ting
      * named values using this segment (_INITMULTI, _TPUTVAL, _TGETVAL)
      * and creating an additional thread (_PGENTRY, _CREATETHREAD).
      *
      * Now down to the procedure division.
      *
      *************************************************************************
      *
       WORKING-STORAGE SECTION.
      *
       01  c-nn.
           02  c-n0 pic x(3) value X"000001".
           02  filler redefines c-n0.
               03  c-0w    pic 9(4) comp.
               03  mem-allocate redefines c-0w pic 9(4) comp.
           02  filler redefines c-n0.
               03  filler  pic x.
               03  c-1w    pic 9(4) comp.
               03  mem-deallocate redefines c-1w pic 9(4) comp.
           02  filler redefines c-n0.
               03  filler pic x.
               03  c-0 pic 9(2) comp.
               03  c-1 pic 9(2) comp.
           02  c-4     pic 9(2) comp value 4.
      *
      *************************************************************************
      *
       01  Concurrent-Variables.
           03  Communications-Area-Ptr     USAGE IS POINTER.
           03  Comms-Area-Ptr-X REDEFINES Communications-Area-Ptr
                                           PIC X(4).
           03  Command-Line-Pointer        USAGE IS POINTER.
           03  Command-Line-Buffer         PIC X(128).
           03  Co-Stat                     PIC X COMP-X VALUE 68.
           03  Key-Area.
               05  Shift-State-1           PIC X COMP-X.
               05  Shift-State-2           PIC X COMP-X.
           03  Memory-Ptr                  USAGE IS POINTER.
           03  Memory-Size                 PIC XXXX COMP-X.
           03  Message-Segment-Size        PIC XX COMP-5 VALUE 2048.
           03  Message-Name.
               05  Message-Name-Size       PIC X COMP-X VALUE 19.
               05  Message-Name-Value      PIC X(19)
                                       VALUE "COMMUNICATIONS-AREA".
           03  Message-Value-Pointer       USAGE IS POINTER.
           03  New-Thread-ID               PIC XXXX COMP-5 VALUE 0.
           03  OS-Information.
               05  OS-Parameter-Size       PIC XX COMP-X VALUE 13.
               05  OS-Type                 PIC X COMP-X.
               05  OS-Version              PIC XXXX COMP-X.
               05  OS-DBCS-Support         PIC X COMP-X.
               05  OS-Char-Coding          PIC X COMP-X.
               05  OS-Country-Code         PIC XX COMP-X.
               05  OS-Code-Page            PIC XX COMP-X.
           03  Screen-Parameters.
               05  Screen-Area-Length      PIC 9(4) COMP-X.
               05  Screen-Area-Start       PIC 9(4) COMP-X.
               05  Screen-Buffer-Start     PIC 9(4) COMP-X VALUE 1.
           03  Segment-Id                  PIC XXXX COMP-X.
           03  Shift-State                 PIC X COMP-X.
           03  Special                     PIC X COMP-X.
           03  Stack-Size                  PIC XX COMP-5 VALUE 1500.
           03  Target-Thread-ID            PIC XXXX COMP-5.
           03  Thread-ID                   PIC XXXX COMP-5 VALUE 0.
           03  Thread-Status               PIC XX COMP-X VALUE 0.
           03  User-Prog-Switch-L          PIC X COMP-X VALUE 0.
               78  ALT-Left-Off            VALUE 0.
               78  ALT-Left-On             VALUE 1.
           03  User-Prog-Switch-R          PIC X COMP-X VALUE 0.
               78  ALT-Right-Off           VALUE 0.
               78  ALT-Right-On            VALUE 1.
           03  User-Prog-Switch-CA         PIC X COMP-X VALUE 0.
               78  CTRL-ALT-Off            VALUE 0.
               78  CTRL-ALT-On             VALUE 1.
           03  W-Stop-Thread               PIC XX COMP-5 VALUE H"0004".
           03  W-Kill-Thread               PIC XX COMP-5 VALUE H"0008".
           03  W-Suspend-Thread            PIC XX COMP-5 VALUE H"0040".
           03  W-Restart-Thread            PIC XX COMP-5 VALUE H"0080".

       78  Thread-Max                      VALUE 3.
       01  Thread-Area                     VALUE LOW-VALUES.
           03  Thread-Chain-Start          PIC X COMP-X.
           03  Thread-Chain-End            PIC X COMP-X.
           03  Thread-Limit                PIC X COMP-X.

           03  Thread-Current              PIC X COMP-X.
           03  Thread-Save                 PIC X COMP-X.
           03  Threads-Running             PIC X COMP-X.

           03  Thread-Cell                 OCCURS Thread-Max.
               05  TC-Entry-Flag           PIC X COMP-X.
                   78  TC-Entry-Free       VALUE 0.
                   78  TC-Entry-Used       VALUE 1.
               05  TC-Chain-Next           PIC X COMP-X.
               05  TC-Chain-Prev           PIC X COMP-X.
               05  TC-Thread-ID            PIC XXXX COMP-5.

       01  Main-Text.
           03  Main-Text-1                 PIC X(80) VALUE
                               "
      -                        "Ŀ
      -                        "".
           03  Main-Text-2                 PIC X(80) VALUE
                               " Concurrent Thread Demonstration - Thre
      -                        "ad 0                                   
      -                        "".
           03  Main-Text-3                 PIC X(80) VALUE
                               "
      -                        "                                       
      -                        "".
           03  Main-Text-4                 PIC X(80) VALUE
                               "
      -                        "
      -                        "".
      *
       LINKAGE SECTION.
      *
       01  Concurrent-Control.
       78  Con-Area-Start                  VALUE Next.
           03  CC-Access-Flag              PIC X COMP-X.
           03  CC-Area-Status              PIC X COMP-X.
               78  CC-Area-Free            VALUE 0.
               78  CC-Command              VALUE 1.
               78  CC-Status               VALUE 2.
               78  CC-Data                 VALUE 3.
           03  CC-Identifier-To            PIC XXXX COMP-5.
           03  CC-Identifier-From          PIC XXXX COMP-5.
           03  CC-Control-Info             PIC X COMP-X.
      * Commands
               78  CC-Terminate-Thread     VALUE 1.
      * Status
               78  CC-Thread-Terminated    VALUE 1.
      * Data
               78  CC-Data-Read            VALUE 1.
               78  CC-Data-Write           VALUE 2.
      *
           03  CC-Data-Area                PIC X(255).
      *
       78  Con-Area-Size                   VALUE Next - Con-Area-Start.
      *
      *************************************************************************
      *
       PROCEDURE DIVISION.
       ConDemo SECTION.
      *
      * Clear the screen and display text
      *
           CALL X"E4".
           PERFORM Write-Main-Text.
      *
      * Find out our own thread identifier.
      * Note: this routine will be replaced by a call by name routine
      * within COBENV.
      *
           CALL X"91" USING Thread-ID, Co-Stat, Thread-Status.
      *
      * Establish which OS environment we are running in,
      * then ensure multi-threading routines are installed.
      *
           CALL "CBL_GET_OS_INFO" USING OS-Information.
           IF OS-Type = 1 OR
              OS-Type = 2
               CALL "COBENV.DLE"
               ON EXCEPTION
                   MOVE "COBENV.DLE not found" TO Main-Text-3 (3:76)
                   GO TO Ld-Exit
               END-CALL
           ELSE
               CALL "COBENV.DLL"
               ON EXCEPTION
                   MOVE "COBENV.DLE not found" TO Main-Text-3 (3:76)
                   GO TO Ld-Exit
               END-CALL.
      *
      *  Next a routine is called to set up a segment (area of store)
      *  that we can use to establish "named" values. These names can
      *  be established by any thread and the values associated with
      *  the name set (put) by the creating thread and reag (get) by
      *  any thread. These items allow threads to communicate
      *  information without requiring rigid call structures to pass
      *  access to data items between programs. We will see how they
      *  are used a little later.
      *
      *  Initialise communications storage areas
      *
           MOVE 0 TO Segment-Id.
           CALL "_INITMULTI" USING BY REFERENCE Segment-Id,
                                   BY VALUE Message-Segment-Size.
           IF Return-Code NOT = 0
               MOVE "Error - Unable to initialise message segment"
                                       TO Main-Text-3 (3:76)
               GO TO Ld-Exit.
      *
      *  Note the use of BY REFERENCE/VALUE this is important on this
      *  and other calls - you must use the correct type and data item
      *  declaration.
      *  Note: the return code identifies more than reason for failure
      *  and in this and other calls may need to be checked
      *  specifically depending on how each routine is used within the
      *  threads.
      *
      *  We now request allocation of some memory that we are going to
      *  use as a communications area between the threads.
      *
           MOVE Con-Area-Size TO Memory-Size.
           CALL X"D0" USING Memory-Ptr
                            Memory-Size
                            C-0w
                            Mem-Allocate.
           IF Memory-Size = 0
               MOVE "Error - Insufficient free memory for communications
      -             " buffer" TO Main-Text-3 (3:76)
               GO TO Ld-Exit.
      *
      *  The pointer to this area is preserved and then a linkage
      *  section data item is attached to this area enabling us to
      *  access the area using the declared linkage section data names.
      *  Note: linkage section data items used in this way are not
      *  referenced in the procedure division statement as they do not
      *  refer to items passed between programs.
      *  WARNING: a called sub-program that sets up linkage items like
      *  this will have to re-attach to the linkage items on EVERY
      *  re-entry (eg SET ADDRESS OF Thing TO Thingy) so the program
      *  will need to know when it has to do it. The memory area and
      *  pointers are safe.
      *
           SET Communications-Area-Ptr TO Memory-Ptr.
           SET ADDRESS OF Concurrent-Control TO Communications-Area-Ptr.
           MOVE LOW-VALUES TO Concurrent-Control.
      *
      *  Now we have the area which are going to use as a general
      *  thread communications buffer we "put" it into the message
      *  segment as a named item for other threads to read. Of course
      *  the other threads need to know your naming convention and ours
      *  is called "COMMUNICATIONS-AREA". What we are doing here is
      *  setting a value to be attached to the name and in this case we
      *  are setting the value to a pointer which will be used by the
      *  other threads to access the area of memory we have acquired.
      *  You don't have to set them to pointers, they could be used for
      *  other information although you are restricted to four bytes.
      *
           CALL "_TPUTVAL" USING BY REFERENCE Message-Value-Pointer,
      *                          BY VALUE Communications-Area-Ptr,
      *
      *  Commented out as a problem with INT code and BY VALUE Pointer
      *  to be removed when fixed
                                 BY VALUE Comms-Area-Ptr-X,
                                 BY REFERENCE Message-Name.
           IF Return-Code NOT = 0
               MOVE "Error - Unable to set communications buffer name"
                                       TO Main-Text-3 (3:76)
               GO TO Ld-Exit.
      *
      *  When a thread is created you have to tell it to start
      *  executing some particular piece of code by supplying a pointer
      *  to the program entry point. For the purpose of this
      *  demonstration we are going to start the thread executing the
      *  normal RTS command line processing code. A command line is set
      *  up containing the program and its parameters prior to creating
      *  the thread. The entry point required is called _EXECTHREAD.
      *
           CALL "_PGENTRY" USING Command-Line-Pointer,
                                 "_EXECTHREAD ".
           IF Return-Code NOT = 0
               MOVE "Error - Unable to get command line entry point"
                                       TO Main-Text-3 (3:76)
               GO TO Ld-Exit.
      *
      *  Now we start a thread using the command line shown below. This
      *  requests the RTS to run the program CONDEMOA in the thread
      *  providing CONDEMOC as the command line to be provided to it,
      *  ie. its parameters.
      *  The program CONDEMOA uses the command line to tell it what to
      *  run - in this case CONDEMOC (TICTAC with an EXIT PROGRAM
      *  added to provide re-entry to CONDEMOA at its termination).
      *
           MOVE "CONDEMOA CONDEMOC" TO Command-Line-Buffer.
           PERFORM Thread-Get-Free-Entry.
      *
      *  The above routine just provides a table of threads that are
      *  active to assist in the closedown process.
      *
           IF Thread-Current NOT = 0
               PERFORM Thread-Add-To-Chain
               MOVE TC-Entry-Used TO TC-Entry-Flag (Thread-Current)
      *
      *  Now set the command line and create the new thread
      *
               DISPLAY Command-Line-Buffer UPON COMMAND-LINE
               CALL "_CREATETHREAD" USING BY VALUE Command-Line-Pointer,
                                          BY REFERENCE New-Thread-Id,
                                          BY VALUE Stack-Size
               IF Return-Code = 0
                   MOVE New-Thread-Id TO TC-Thread-ID (Thread-Current)
      *
      *  The thread is running so we preserve the returned identifier
      *  for identification of communications via the named area.
      *
               ELSE
                   PERFORM Thread-Delete-From-Chain
                   MOVE "Unable to create new thread"
                                       TO Main-Text-3 (3:76)
                   GO TO Ld-Exit
               END-IF
           ELSE
               MOVE "Error - No free slots to create thread"
                                       TO Main-Text-3 (3:76)
               GO TO Ld-Exit
           END-IF.
      *
      *
      *  This just runs CONDEMOB in an additional thread that provides
      *  a clock near the top right of the screen.
      *
           MOVE "CONDEMOB" TO Command-Line-Buffer.
           PERFORM Thread-Get-Free-Entry.
           IF Thread-Current NOT = 0
               PERFORM Thread-Add-To-Chain
               MOVE TC-Entry-Used TO TC-Entry-Flag (Thread-Current)
               DISPLAY Command-Line-Buffer UPON COMMAND-LINE
               CALL "_CREATETHREAD" USING BY VALUE Command-Line-Pointer,
                                          BY REFERENCE New-Thread-Id,
                                          BY VALUE Stack-Size
               IF Return-Code = 0
                   MOVE New-Thread-Id TO TC-Thread-ID (Thread-Current)
               ELSE
                   PERFORM Thread-Delete-From-Chain
                   MOVE "Unable to create new thread"
                                       TO Main-Text-3 (3:76)
                   GO TO Ld-Exit
               END-IF
           ELSE
               MOVE "Error - No free slots to create thread"
                                       TO Main-Text-3 (3:76)
               GO TO Ld-Exit
           END-IF.
      *
      *  So that's all there is to it - well starting them up at least.
      *  Have a look at CONDEMOA and CONDEMOB to see how they pick up
      *  the information.
      *
      **************************************************************************
      *
      *  This program now loops waiting for the other two to terminate
      *  and to pass the time will display a message if certain
      *  combination of shift state keys are detected.
      *
      *
       Task-Loop.
           CALL "_KSTATE" USING Key-Area.
           MOVE Shift-State-1 TO Shift-State
           CALL X"C1" USING Shift-State, C-4.
      * Check for ALT/Left Shift
           IF Shift-State = 160
               MOVE ALT-Right-Off TO User-Prog-Switch-R
               MOVE CTRL-ALT-Off TO User-Prog-Switch-CA
               IF User-Prog-Switch-L = ALT-Left-Off
                   MOVE ALT-Left-On TO User-Prog-Switch-L
                   MOVE "Alt + Left shift detected"
                                    TO Main-Text-3 (3:76)
                   PERFORM Write-Main-Text
               END-IF
           ELSE
      * Check for ALT/Right Shift
               IF Shift-State = 144
                   MOVE ALT-Left-Off TO User-Prog-Switch-L
                   MOVE CTRL-ALT-Off TO User-Prog-Switch-CA
                   IF User-Prog-Switch-R = ALT-Right-Off
                       MOVE ALT-Right-On TO User-Prog-Switch-R
                       MOVE "Alt + Right shift detected"
                                    TO Main-Text-3 (3:76)
                       PERFORM Write-Main-Text
                   END-IF
               ELSE
                   MOVE ALT-Right-Off TO User-Prog-Switch-R
                   MOVE ALT-Left-Off TO User-Prog-Switch-L
      * Check for CTRL/ALT
                   IF Shift-State = 192
                       IF User-Prog-Switch-CA = CTRL-ALT-Off
                           MOVE CTRL-ALT-On TO User-Prog-Switch-CA
                           MOVE "Alt + Ctrl detected"
                                    TO Main-Text-3 (3:76)
                           PERFORM Write-Main-Text
                       END-IF.
      *
      *  As the user interface program is CONDEMOC (TICTAC) when this
      *  terminates, the clock thread is asked to terminate and
      *  the system closes down in an orderly manner.
      *  CONDEMOC informs us that it is closing down using the shared
      *  communications area. On receipt of this message we send a
      *  termination request to the clock thread. See Check-Interface.
      *  When both threads have gone, or at least said they are going
      *  then we terminate.
      *
           PERFORM Check-Interface.
      *
      *  Release the remainder of this timeslice.
      *
           CALL "_COYIELD".
      *
           IF Threads-Running NOT = 0
               GO TO Task-Loop.
      *
           SET Memory-Ptr TO Communications-Area-Ptr
           MOVE 0 TO Memory-Size.
           CALL X"D0" USING Memory-Ptr
                            Memory-Size
                            C-0w
                            Mem-Deallocate.
      *
       Ld-Exit.
           PERFORM Write-Main-Text.
           EXIT PROGRAM.
           STOP RUN.
      *
      *-----------------------------------------------------------------
      *
       Check-Interface SECTION.
           ADD 1 TO CC-Access-Flag.
           IF CC-Access-Flag = 1 AND
              CC-Identifier-To = Thread-ID AND
              CC-Area-Status = CC-Status
               PERFORM Thread-Get-From-Entry
               MOVE CC-Area-Free TO CC-Area-Status
               IF Thread-Current = 0
      *        Thread has already been deleted from chain
               ELSE
                   EVALUATE CC-Control-Info
                   WHEN CC-Thread-Terminated
                       PERFORM Thread-Delete-From-Chain
                       IF Threads-Running NOT = 0
                           MOVE Thread-Chain-Start TO Thread-Current
                           MOVE CC-Command TO CC-Area-Status
                           MOVE CC-Terminate-Thread TO CC-Control-Info
                           MOVE TC-Thread-ID (Thread-Current)
                                                    TO CC-Identifier-To
                           MOVE Thread-ID TO CC-Identifier-From
                   END-EVALUATE
               END-IF
           END-IF.
           SUBTRACT 1 FROM CC-Access-Flag.
      *
      **************************************************************************
      *
      *  These routines are not used in this example program but show
      *  how the call parameters should be set.
      *
       Kill-Thread SECTION.
           MOVE TC-Thread-ID (Thread-Current) TO Target-Thread-ID.
           CALL "_SIGNALTHREAD" USING BY VALUE Target-Thread-ID,
                                      BY VALUE W-Kill-Thread.

       Restart-Thread SECTION.
           MOVE TC-Thread-ID (Thread-Current) TO Target-Thread-ID.
           CALL "_SIGNALTHREAD" USING BY VALUE Target-Thread-ID,
                                      BY VALUE W-Restart-Thread.

       Suspend-Thread SECTION.
           MOVE TC-Thread-ID (Thread-Current) TO Target-Thread-ID.
           CALL "_SIGNALTHREAD" USING BY VALUE Target-Thread-ID,
                                      BY VALUE W-Suspend-Thread.

       Stop-Thread SECTION.
           MOVE TC-Thread-ID (Thread-Current) TO Target-Thread-ID.
           CALL "_SIGNALTHREAD" USING BY VALUE Target-Thread-ID,
                                      BY VALUE W-Stop-Thread.

      *-----------------------------------------------------------------
       Thread-Add-To-Chain SECTION.
           IF Thread-Chain-Start = 0
               MOVE Thread-Current TO Thread-Chain-Start
               MOVE Thread-Current TO Thread-Chain-End
           ELSE
               MOVE Thread-Current TO TC-Chain-Next (Thread-Chain-End)
               MOVE Thread-Chain-End TO TC-Chain-Prev (Thread-Current)
               MOVE Thread-Current TO Thread-Chain-End.
           ADD 1 TO Threads-Running.

      *-----------------------------------------------------------------
       Thread-Delete-From-Chain SECTION.
           IF Thread-Current = Thread-Chain-Start
               MOVE TC-Chain-Next (Thread-Current) TO Thread-Chain-Start
               IF Thread-Chain-Start = 0
                   MOVE 0 TO Thread-Chain-End
               END-IF
           ELSE
           IF Thread-Current = Thread-Chain-End
               MOVE TC-Chain-Prev (Thread-Current) TO Thread-Chain-End
               MOVE 0 TO TC-Chain-Next (Thread-Chain-End)
           ELSE
               MOVE TC-Chain-Prev (Thread-Current) TO Special
               MOVE TC-Chain-Next (Thread-Current)
                                       TO TC-Chain-Next (Special)
               MOVE TC-Chain-Next (Thread-Current) TO Special
               MOVE TC-Chain-Prev (Thread-Current)
                                       TO TC-Chain-Prev (Special).
           MOVE LOW-VALUES TO Thread-Cell (Thread-Current).
           SUBTRACT 1 FROM Threads-Running.

      *-----------------------------------------------------------------
       Thread-Get-Free-Entry SECTION.
           MOVE 1 TO Thread-Current.
       Thread-Get-Free-Next.
           IF TC-Entry-Flag (Thread-Current) NOT = TC-Entry-Free
               IF Thread-Current NOT = Thread-Limit
                   ADD 1 TO Thread-Current
                   GO TO Thread-Get-Free-Next
               ELSE
                   MOVE 0 TO Thread-Current.

      *-----------------------------------------------------------------
       Thread-Get-From-Entry SECTION.
           MOVE Thread-Chain-Start TO Thread-Current.
       Thread-Get-From-Next.
           IF TC-Thread-ID (Thread-Current) = CC-Identifier-From
           ELSE
               IF Thread-Current NOT = Thread-Chain-End
                   MOVE TC-Chain-Next (Thread-Current) TO Thread-Current
                   GO TO Thread-Get-From-Next
               ELSE
                   MOVE 0 TO Thread-Current.

      *-----------------------------------------------------------------
      *
       Write-Main-Text SECTION.
           MOVE 21 TO Screen-Area-Start.
           MULTIPLY 80 BY Screen-Area-Start.
           ADD 1 TO Screen-Area-Start.
           MOVE 320 TO Screen-Area-Length.
           CALL X"B7" USING C-1, Screen-Parameters, Main-Text.
      *
      *************************************************************************
