#include    <stdio.h>

/*******************************************************************/
/*								   */
/*		(C) Micro Focus Ltd. 1989			   */
/*								   */
/*		       EXTFHRD.C				   */
/*								   */
/* Example C program to sequentially read a COBOL ISAM file.	   */
/* Must be compiled with -Zp swtich to prevent word alignment of   */
/* FCD structure and in large memory model, -Al, for Microsoft C   */
/* Version 5.1.  */

#define USHORT unsigned short
#define ULONG  unsigned long
#define UCHAR  unsigned char

/**********************************************************************/
/*								      */
/*    FILE ACCESS QUALIFIER					      */
/*								      */
/**********************************************************************/

#define    OPEN_INPUT		   0X00
#define    CLOSE		   0X80
#define    READ_NEXT_NO_LOCK	   0X8D

/**********************************************************************/
/*								      */
/*	FILE CONTROL DESCRIPTOR 				      */
/*								      */
/*	    "EXTFH" INTERFACE AREA. (LENGTH = 100 BYTES)	      */
/*								      */
/*								      */
/**********************************************************************/
struct FCD {
	char status[2]; 		/* FILE STATUS			*/
	char filler1[3];
	UCHAR organization;		 /* ORGANIZATION		*/
	UCHAR access;			 /* ACCESS MODE 		*/
	UCHAR open_mode;
	char filler2[3];
	USHORT name_length;		 /* FILE NAME LENGTH		*/
	char filler3[11];
	UCHAR lock_mode;		 /* LOCK MODE			*/
	UCHAR optional; 		 /* OPTINAL FLAG		*/
	char filler4[3];
	char handle[4];
	UCHAR status_type;		 /* ANSI FLAG			*/
	UCHAR file_format;
	char filler5[3];
	USHORT reclen_maximum;	       /* MAX RECORD LENGTH		*/
	char filler6[3];
	ULONG relative_key;		 /* RELATIVE KEY VALUE		*/
	UCHAR recording_mode;		 /* RECORDING MODE		*/
	USHORT reclen_current;		 /* CURRENT RECORD LENGTH	*/
	USHORT reclen_minimum;		  /* MINIMUM RECORD LENGTH	*/
	USHORT key_id;
	USHORT key_length;		 /* KEY LENGTH			*/
	char far *record_pointer;	     /* RECORD AREA POINTER	*/
	char far *name_pointer; 	    /* FILE NAME POINTER	*/
	char far *keydef_pointer;	    /* KEY DEFINE AREA POINTER	*/
	char filler7[32];
};
/**********************************************************************/
/*								      */
/*	GLOBAL KEY INFORMATION					      */
/*		(LENGTH = 14 BYTES)				      */
/*								      */
/**********************************************************************/
struct KEY_GLOBAL {
	char keydef_length[2];		/* LENGTH OF KEY DEFINITION BLOCK */
	char version_number_g;		/* RESERVED FOR VERSION NUMBER	*/
	char index_format;		/* RESERVED FOR INDEX FORMAT	*/
	char integrity_level_g; 	/* RESERVED FOR INTEGRITY LEVEL */
	char tuning_flags;		/* RESERVED FOR TUNING FLAGS	*/
	char key_count[2];		/* NUMBER OF KEYS		*/
	char reserved_index_areas[2];	/* RESERVED FOR RESERVED INDEX AREA */
	char index_reclen[4];		/* RESERVED FOR INDEX RECORD LENGTH */

/**********************************************************************/
/*								      */
/*	KEY DEFINITIONS 					      */
/*		(LENGTH = 16 BYTES)				      */
/*								      */
/**********************************************************************/

struct KEY_DEF_BIT {
	char component_count[2]; /* COMPONENT COUNT		 */
	char component_defs[2];  /* OFFSET TO FIRST COMPONENT	 */
	char key_flags;
	char key_compression;
	char sparse_characters[2];
	char key_password[8];
} kd;

/**********************************************************************/
/*								      */
/*	COMPONENT DEFINITIONS BIT				      */
/*		(LENGTH = 10 BYTES)				      */
/*								      */
/**********************************************************************/
struct COMP_DEF_BIT {
	char component_flag;  /* COMPONENT FLAGS (=0) */
	char component_type;  /* COMPONENT TYPE  (=0) */
	char component_offset[4]; /* COMPONENT OFFSET	  */
	char component_length[4]; /* COMPONENT LENGTH	  */
    } cd;
};	      /* end of KEY_GLOBAL struc */


extern far pascal extfh();  /* necessary evil.	COBOL does not put */
			    /* underscore on function names but    */
			    /* does follow C calling convention.   */
			    /* pascal keyword suppresses the under */
			    /* score but now arguments must be	   */
			    /* reveresed when calling extfh and    */
			    /* since COBOL expects the caller to   */
			    /* clean the stack this will cause 4   */
			    /* bytes of garbage to accumulate on   */
			    /* the stack for every call to extfh.  */

struct	FCD	    fcd = {0};	  /* insure all reserved fields are zero */
struct	KEY_GLOBAL  kg = {0};	  /* insure all reserved fields are zero */

char	op_code[]  = {0XFA, 'Q', 0};
char	filename[] = "FTEST";
char	rec_data[49] = " ";

main()
{
    int     i;
    char    *dc;
    int     j;

    /* initialize required fields in FCD */

    fcd.organization	  = 2;		    /* INDEXED */
    fcd.open_mode	  = 0x80;	    /* mark file as closed */
    fcd.access		  = 0x04;	    /* RANDOM */
    fcd.name_length	  = 0x0500;	    /* 5 bytes COBOL COMP format */
    fcd.lock_mode	  = 0x00;	    /* EXCLUSIVE */
    fcd.optional	  = 0;		    /* Optional  */
    fcd.reclen_maximum	  = 0x3100;	 /* 49 bytes COBOL COMP format */
    fcd.reclen_minimum	  = 0x3100;	 /* 49 bytes COBOL COMP format */
    fcd.recording_mode	  = 0;		    /* FIXED LENGTH */
    fcd.record_pointer	  = (char far *)rec_data;  /* record pointer	*/
    fcd.name_pointer	  = (char far *)filename;  /* file name pointer */
    fcd.keydef_pointer	  = (char far *)&kg;	   /* key define area	*/
    fcd.file_format	     = 0;		   /* current ISAM struc */

    /* Initialize required global key structure fields */

    kg.keydef_length[0]   = 0;
    kg.keydef_length[1]   = sizeof(struct KEY_GLOBAL);
    kg.key_count[0]	  = 0;
    kg.key_count[1]	  = 1;		    /* Only primary key */

    /* Initialize required key definition structure fields */

    kg.kd.component_count[0] = 0;
    kg.kd.component_count[1] = 1;    /* only one field in this key */
    kg.kd.component_defs[0]  = 0;
    kg.kd.component_defs[1]  = 30;
    kg.kd.key_flags	     = 0x10;	       /* Primary key */
    kg.kd.key_compression    = 0;

    kg.cd.component_flag      = 0;
    kg.cd.component_type      = 0;
    kg.cd.component_offset[0] = 0;
    kg.cd.component_offset[1] = 0;
    kg.cd.component_offset[2] = 0;
    kg.cd.component_offset[3] = 0;		/* Primary key offset */
    kg.cd.component_length[0] = 0;
    kg.cd.component_length[1] = 0;
    kg.cd.component_length[2] = 0;
    kg.cd.component_length[3] = 29;		 /* Key length = 29 */


    op_code[1] = OPEN_INPUT;
    extfh(&fcd, op_code);
    printf("File status after open ");
    printf("status=%x %x\n",fcd.status[0],fcd.status[1]);

    op_code[1] = READ_NEXT_NO_LOCK;

    for( i=0; i<3; i++) {
	extfh(&fcd, op_code);
	printf("File status after read #%d ",i);
	printf("status=%x %x\n",fcd.status[0],fcd.status[1]);
	printf("Data record =");
	for(j = 29, dc = &rec_data[29]; j < sizeof(rec_data); j++)
	    printf( "%c", *dc++ );
	printf("\n");
    }

    op_code[1] = CLOSE;
    extfh(&fcd, op_code);
    printf("File status after close ");
    printf("status=%x %x\n",fcd.status[0],fcd.status[1]);
};
